/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
* This file is part of the libgltf project.
*
* This Source Code Form is subject to the terms of the Mozilla Public
* License, v. 2.0. If a copy of the MPL was not distributed with this
* file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

#include "libgltf.h"
#include "RenderScene.h"

glTFHandle* gltf_renderer_init(const glTFFile* jsonfile)
{
    RenderScene* renderScene = new RenderScene();
    glTFHandle* gltf_result = renderScene->initScene(jsonfile);
    gltf_result->renderer = renderScene;
    return gltf_result;
}

int gltf_renderer_set_content(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->initRender();
}

void gltf_render_FPS_enable(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->initFPS();
}

int gltf_prepare_renderer(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->prepareRender(&handle->viewport);
}

void gltf_renderer(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->render();
    return;
}

void gltf_complete_renderer(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->completeRender();
    return;
}

void gltf_renderer_release(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->releaseRender();
    delete renderScene;
    return;
}

glm::vec3* gltf_get_model_center_pos(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->getModelCenterPos();
}

void gltf_get_camera_pos(glTFHandle* handle, glm::vec3* pos,
                         glm::vec3* view, glm::vec3* up)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->getCameraPos(pos, view, up);
    return;
}

double gltf_get_model_size(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->getModelSize();
}

void gltf_renderer_move_camera(glTFHandle* handle, double x, double y,
                               double z, double time)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->getCamera()->moveCamera(x, y, z, time);
    return;
}

void gltf_renderer_rotate_model(glTFHandle* handle, double horizontal,
                                double vertical, double planar)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->getCamera()->RotateObjectMouse(horizontal, vertical, planar);
    return;
}

void gltf_renderer_rotate_camera(glTFHandle* handle, double horizontal,
                                 double vertical, double planar)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->getCamera()->RotateCamera(horizontal, vertical, planar);
    renderScene->getCamera()->rotateCamera(horizontal, vertical, planar);
    return;
}

void gltf_renderer_fly_camera(glTFHandle* handle, glm::vec3 glPosInfo,
    double time)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->renderFlyCamera(glPosInfo,time);
}

void gltf_renderer_stop_rotate_model(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->getCamera()->removeRotateObject();
    return;
}

void gltf_enable_rotation(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->EnableRotation();
}

void gltf_disable_rotation(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->DisableRotation();
}

void gltf_enable_transparency(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->EnableTransparency();
}

void gltf_disable_transparency(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->DisableTransparency();
}

int gltf_renderer_get_bitmap(glTFHandle** handle, int size,
                             char* buffer, GLenum format, double time)
{

    if (size <= 0)
        return LIBGLTF_INVALID_SIZE;
    RenderScene* renderScene = (RenderScene*)handle[0]->renderer;
    int status = renderScene->prepareRenderBitmap(&(handle[0]->viewport));
    if (LIBGLTF_SUCCESS != status)
    {
        return status;
    }
    for (int i = 0; i < size; i++)
    {
        RenderScene* tRenderScene = (RenderScene*)handle[i]->renderer;
        tRenderScene->renderBitmap(time);
    }
    renderScene->completeRenderBitmap(&(handle[0]->viewport),
                                      (unsigned char*)buffer, format);
    return LIBGLTF_SUCCESS;
}

void gltf_animation_start(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->startAnimation();
    return;
}

void gltf_animation_stop(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->stopAnimation();
    return;
}

void gltf_orbit_view_stop(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->stopAerialView();
    return;
}

void gltf_orbit_mode_start(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->startAerialView();
    return;
}

int gltf_animation_is_playing(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->isAnimPlay();
}

void gltf_animation_set_looping(glTFHandle* handle, int loop)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->setAnimLoop(loop);
    return;
}

int gltf_animation_get_looping(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->getAnimLoop();
}

double gltf_animation_get_duration(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->getAnimDuration();
}

void gltf_animation_set_time(glTFHandle* handle, double time)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    if (time <= 0)
    {
        renderScene->setAnimTime(0.0);
    }
    else
    {
        renderScene->setAnimTime(time);
    }
    return;
}

double gltf_animation_get_time(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->getAnimTime();
}

void gltf_animation_resume(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->resumeAnim();
    return;
}

void gltf_get_camera_info(glTFHandle* handle, cameraInfo* pCameraInfo)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->getCameraInfo(pCameraInfo);
    return;
}

/*GLdouble* get_glpos_info(glTFHandle* gltfHandle, int x, int y)
{
    GLint viewPort[4];
    GLdouble modelView[16];
    GLdouble projection[16];
    GLfloat winX, winY, winZ;
    static GLdouble pos[3];

    viewPort[0] = gltfHandle->viewport.x;
    viewPort[1] = gltfHandle->viewport.y;
    viewPort[2] = gltfHandle->viewport.width;
    viewPort[3] = gltfHandle->viewport.height;

    RenderScene* scene = (RenderScene*)gltfHandle->renderer;
    glm::mat4& viewMatrix = scene->getCamera()->getLook();
    for (int i = 0; i < 4; ++i)
    {
        for (int j = 0; j < 4; ++j)
        {
            modelView[i * 4 + j] = viewMatrix[i][j];
        }
    }

    glm::mat4 projectionMatrix = scene->getCamera()->getPerspective();
    for (int i = 0; i < 4; ++i)
    {
        for (int j = 0; j < 4; ++j)
        {
            projection[i * 4 + j] = projectionMatrix[i][j];
        }
    }

    winX = (float)x;
    winY = (float)viewPort[3] - (float)y;
    glEnable(GL_DEPTH_TEST);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glReadPixels(winX, (int)winY, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &winZ);

    gluUnProject(winX, winY, winZ, modelView, projection, viewPort, &pos[0], &pos[1], &pos[2]);

    return pos;
}*/

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
